package controllers

import (
	"aon_app_server/models"
	"aon_app_server/utils/mongo"
	"bytes"
	"encoding/json"
	"github.com/beego/beego/v2/core/logs"
	beego "github.com/beego/beego/v2/server/web"
	"go.mongodb.org/mongo-driver/bson"
	"io"
	"net/http"
	"time"
)

type TaskController struct {
	MainController
}

var execTasks = make(chan *models.Task, 100)

// Prediction @router /prediction/ [post]
func (server *TaskController) Prediction() {
	body := server.Ctx.Input.RequestBody
	task := models.Task{}
	err := json.Unmarshal(body, &task) //解析body中数据
	logs.Debug("appRequest", task)
	if err != nil {
		server.respond(models.NoRequestBody, err.Error())
		return
	}

	task.CreatedTime = time.Now().UTC()
	id, err := mongo.Insert(&task)
	if err != nil {
		server.respond(models.BusinessFailed, err.Error())
		return
	}
	task.Id = id
	//execTasks <- &task
	result, err := sendTask(&task)
	if err != nil {
		server.respond(http.StatusOK, err.Error())
		return
	}
	server.respond(http.StatusOK, "", result)
}

func copyImages(images []string) []string {
	imagesToCopy := models.ImagesToCopy{
		Sources: images,
	}
	url, _ := beego.AppConfig.String("imageUrl")
	payload := new(bytes.Buffer)
	json.NewEncoder(payload).Encode(imagesToCopy)
	resp, err := http.Post(url, "application/json;charset=UTF-8", payload)
	if err != nil {
		logs.Info("Error sending request:", err)
		return images
	}
	defer resp.Body.Close()
	logs.Info("copyImages resp code", resp.StatusCode)
	body, err := io.ReadAll(resp.Body)
	if err != nil {
		logs.Info("Error reading response:", err)
		return images
	}
	logs.Debug("copyImages body", string(body))

	var response []models.ImageCopied
	if err := json.Unmarshal(body, &response); err != nil {
		return images
	}
	var backImages []string
	for _, value := range response {
		if value.Code == 200 {
			backImages = append(backImages, value.Data)
		}
	}
	if len(backImages) > 0 {
		return backImages
	}
	return images
}

func sendTask(task *models.Task) (*models.TaskResponse, error) {
	host, _ := beego.AppConfig.String("taskUrl")
	url := host + task.ApiPath
	payload := new(bytes.Buffer)
	//var input interface{}
	//if err := json.Unmarshal([]byte(task.Input), &input); err != nil {
	//	setError(task, "task.Input Unmarshal error:"+err.Error())
	//	logs.Info("sendTask task.Input Unmarshal response:", err)
	//	return nil, err
	//}
	json.NewEncoder(payload).Encode(task.Input)
	client := &http.Client{}
	request, err := http.NewRequest("POST", url, payload)
	if err != nil {
		setError(task, "sendTask request create error:"+err.Error())
		logs.Info("sendTask Error NewRequest request:", err)
		return nil, err
	}
	apikey, _ := beego.AppConfig.String("apikey")
	request.Header.Add("apikey", apikey)

	logs.Info("sendTask client sending request:")
	resp, err := client.Do(request)
	if err != nil {
		setError(task, "Task sending error:"+err.Error())
		logs.Info("sendTask Error sending request:", err)
		return nil, err
	}
	defer resp.Body.Close()
	logs.Info("sendTask resp code", resp.StatusCode)
	body, err := io.ReadAll(resp.Body)
	logs.Info("sendTask body", string(body))
	if resp.StatusCode == 200 && body == nil {
		setError(task, "Task response body null")
		logs.Info("sendTask Body reading response:", err)
		return nil, err
	}
	if err != nil {
		setError(task, "Task read response body error:"+err.Error())
		logs.Info("sendTask Error reading response:", err)
		return nil, err
	}
	var response models.TaskResponse
	if err = json.Unmarshal(body, &response); err != nil {
		setError(task, "Task response Unmarshal error:"+err.Error())
		logs.Info("sendTask Error Unmarshal response:", err)
		return nil, err
	}
	taskId := resp.Header.Get("task-id")
	task.TaskId = taskId
	task.UpdatedTime = time.Now().UTC()
	if response.Task.IsSuccess {
		task.Status = 2
		task.Output = copyImages(response.Output)
		mongo.Update(task)
		response.Output = task.Output

		//data, err := json.Marshal(response.Output)
		//if err != nil {
		//	logs.Info("sendTask Output Unmarshal response:", err)
		//	//return nil, err
		//	task.Error = "Output json Unmarshal error"
		//	mongo.Update(task)
		//} else {
		//	task.Output = string(data)
		//	mongo.Update(task)
		//}
	} else {
		task.Status = 3
		task.Error = response.Task
		_, err = mongo.Update(task)
		if err != nil {
			logs.Info("Update Task Error:", err)
		}
		//data, err := json.Marshal(response.Task)
		//if err != nil {
		//	logs.Info("sendTask response.Task Unmarshal response:", err)
		//	//return
		//	task.Error = "response task json Unmarshal error"
		//} else {
		//	task.Error = string(data)
		//}
		//mongo.Update(task)
	}
	return &response, nil
}

func setError(task *models.Task, error string) {
	task.Status = 3
	task.Error = error
	_, err := mongo.Update(task)
	if err != nil {
		logs.Info("Update Task Error:", err)
	}
}

// Result @router /result/:task_id [get]
func (server *TaskController) Result() {
	taskId := server.GetString("task_id")
	data := struct {
		Result string `json:"result"`
		TaskId string `json:"task_id"`
	}{
		Result: "success",
		TaskId: taskId,
	}
	server.respond(http.StatusOK, "", data)
}

// List @router /list/ [post]
func (server *TaskController) List() {
	body := server.Ctx.Input.RequestBody
	request := models.ListRequest{}
	err := json.Unmarshal(body, &request) //解析body中数据
	logs.Debug("Request", string(body), request)
	if err != nil {
		server.respond(models.NoRequestBody, err.Error())
		return
	}

	if request.Page == 0 {
		request.Page = 1
	}
	if request.Size == 0 {
		request.Size = 10
	}

	total, data, err := mongo.Query("Task", request.Page, request.Size, request.Filter)
	if err != nil {
		logs.Info("List Error:", err)
	}
	var tasks []models.Task
	for _, bsonD := range data {
		var task models.Task
		// 将 bson.D 转换为 bson.Raw
		bsonRaw, err := bson.Marshal(bsonD)
		if err != nil {
			server.respond(models.BusinessFailed, err.Error())
		}
		// 将 bson.Raw 解码为 User 结构体
		err = bson.Unmarshal(bsonRaw, &task)
		if err != nil {
			server.respond(models.BusinessFailed, err.Error())
		}
		if task.Input != nil {
			raw, err := bson.Marshal(task.Input)
			if err != nil {
				server.respond(models.BusinessFailed, err.Error())
			}
			// 将 bson.Raw 解码为 User 结构体
			var input map[string]interface{}
			err = bson.Unmarshal(raw, &input)
			if err != nil {
				server.respond(models.BusinessFailed, err.Error())
			}
			task.Input = input
		}

		if task.Error != nil {
			raw, err := bson.Marshal(task.Error)
			if err != nil {
				server.respond(models.BusinessFailed, err.Error())
			}
			// 将 bson.Raw 解码为 User 结构体
			var error1 models.TaskReturn
			err = bson.Unmarshal(raw, &error1)
			if err != nil {
				server.respond(models.BusinessFailed, err.Error())
			}
			task.Error = error1
		}

		tasks = append(tasks, task)
	}
	responseData := struct {
		Total int64       `json:"total"`
		Data  interface{} `json:"data,omitempty"`
	}{
		Total: total,
		Data:  tasks,
	}
	server.respond(http.StatusOK, "", responseData)
}
